package com.example.sncaugmentedrealityvirtual_tour.snc_augmented_reality_virtual_tour;

//Activity used to create a new building in the database

import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Base64;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.MalformedInputException;

public class Create_Building extends AppCompatActivity {

    private static int RESULT_LOAD_IMG = 1;
    private ImageView imgView;

    //json data strings
    String json_string1;
    String json_string2;

    //building data variables
    String imgDecodableString = "0";
    String building_type = null;
    private EditText mBuildingName;
    private EditText mBuildingNickname;
    private EditText mBuildingLat;
    private EditText mBuildingLong;
    private EditText mBuildingUrl;
    private EditText mBuildingAddress;
    private EditText mBuildingDescription;



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_create__building);

        //Initializes dropdown to select the building type
        Spinner dropdown = (Spinner)findViewById(R.id.buildingTypes);
        dropdown.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

            //when spinner dropdown is selected
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                //if no building type selected yet
                if (parent.getSelectedItem().toString().equals("")) {
                    //Toast.makeText(Create_Building.this, "Please Select a Building Type", Toast.LENGTH_LONG).show();
                    building_type = "0";
                }
                //if not building type is selected
                else {
                    //if building type selected is Academic
                    if(parent.getSelectedItem().toString().equals("Academic")) {
                        building_type = "1";
                    }
                    //if building type selected is food
                    else if(parent.getSelectedItem().toString().equals("Food")) {
                        building_type = "2";
                    }
                    //if building type selected is Residential
                    else if(parent.getSelectedItem().toString().equals("Residential")) {
                        building_type = "3";
                    }
                    //if building type selected is Miscellaneous
                    else if(parent.getSelectedItem().toString().equals("Miscellaneous")) {
                        building_type = "4";
                    }
                    //Toast.makeText(Create_Building.this, building_type, Toast.LENGTH_LONG).show();
                }
            }

            //auto-generated function
            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        //get json admin and building data
        json_string1 = getIntent().getExtras().getString("json_data");
        json_string2 = getIntent().getExtras().getString("json_building_data");

        //establish each editText field
        mBuildingName = (EditText)findViewById(R.id.building_name1);
        mBuildingNickname = (EditText)findViewById(R.id.building_nick1);
        mBuildingLat = (EditText)findViewById(R.id.building_lat1);
        mBuildingLong = (EditText)findViewById(R.id.building_long1);
        mBuildingUrl = (EditText)findViewById(R.id.building_url1);
        mBuildingAddress = (EditText)findViewById(R.id.building_address1);
        mBuildingDescription = (EditText)findViewById(R.id.building_description1);
    }

    //overrides back button to return to admin main activity
    @Override
    public void onBackPressed(){
        Intent intent = new Intent(this, Admin_Main.class);
        intent.putExtra("json_data", json_string1);
        intent.putExtra("json_building_data", json_string2);
        startActivity(intent);
    }

    //when create button is selected
    public void attemptCreate(View view)
    {
        boolean cancel = false;
        boolean image = false;
        boolean type = false;
        View focusView = null;

        // Reset errors.
        mBuildingName.setError(null);
        mBuildingNickname.setError(null);
        mBuildingLat.setError(null);
        mBuildingLong.setError(null);
        mBuildingAddress.setError(null);
        mBuildingNickname.setError(null);
        mBuildingDescription.setError(null);

        //populates variables with inputted text from the screen
        String name = mBuildingName.getText().toString();
        String nickname = mBuildingNickname.getText().toString();
        String lat = mBuildingLat.getText().toString();
        String log = mBuildingLong.getText().toString();
        String url = mBuildingUrl.getText().toString();
        String address = mBuildingAddress.getText().toString();
        String description = mBuildingDescription.getText().toString();

        //if building name is empty
        if (TextUtils.isEmpty(name)) {
            mBuildingName.setError(getString(R.string.error_field_required));
            focusView = mBuildingName;
            cancel = true;
        }

        //if building nickname is empty
        if (TextUtils.isEmpty(nickname)) {
            mBuildingNickname.setError(getString(R.string.error_field_required));
            focusView = mBuildingNickname;
            cancel = true;
        }

        //if building latitude is empty
        if (TextUtils.isEmpty(lat)) {
            mBuildingLat.setError(getString(R.string.error_field_required));
            focusView = mBuildingLat;
            cancel = true;
        }

        //if building longitude is empty
        if (TextUtils.isEmpty(log)) {
            mBuildingLong.setError(getString(R.string.error_field_required));
            focusView = mBuildingLong;
            cancel = true;
        }

        //if building url is empty
        if (TextUtils.isEmpty(url)) {
            mBuildingUrl.setError(getString(R.string.error_field_required));
            focusView = mBuildingUrl;
            cancel = true;
        }

        //if building address is empty
        if (TextUtils.isEmpty(address)) {
            mBuildingAddress.setError(getString(R.string.error_field_required));
            focusView = mBuildingAddress;
            cancel = true;
        }

        //if building description is empty
        if (TextUtils.isEmpty(description)) {
            mBuildingDescription.setError(getString(R.string.error_field_required));
            focusView = mBuildingDescription;
            cancel = true;
        }

        //if no image is selected
        if (imgDecodableString.equals("0")) {
            image = true;
        }

        //if not building type is selected
        if (building_type.equals("0")) {
            type = true;
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            focusView.requestFocus();
        }
        //if no image is selected
        else if(image) {
            Toast.makeText(Create_Building.this, "Please Select a Building Image", Toast.LENGTH_LONG).show();
        }
        //if no building type is selected
        else if(type) {
            Toast.makeText(Create_Building.this, "Please Select a Building Type", Toast.LENGTH_LONG).show();
        }
        else {

            //inserts a new building into the database
            BackgroundTask backgroundTask = new BackgroundTask(this);
            backgroundTask.execute(name, nickname, lat, log, url, address, description);
        }
    }

    //opens and loads photo gallery so that the user can select a photo for a building
    public void loadImagefromGallery(View view) {
        // Create intent to Open Image applications like Gallery, Google Photos
        Intent galleryIntent = new Intent(Intent.ACTION_PICK,
                android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        // Start the Intent
        startActivityForResult(galleryIntent, RESULT_LOAD_IMG);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        try {
            // When an Image is picked
            if (requestCode == RESULT_LOAD_IMG && resultCode == RESULT_OK
                    && null != data) {
                // Get the Image from data

                Uri selectedImage = data.getData();
                String[] filePathColumn = { MediaStore.Images.Media.DATA };

                // Get the cursor
                Cursor cursor = getContentResolver().query(selectedImage,
                        filePathColumn, null, null, null);
                // Move to first row
                cursor.moveToFirst();

                int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
                imgDecodableString = cursor.getString(columnIndex);
                cursor.close();


            } else {
                Toast.makeText(this, "You haven't picked Image",
                        Toast.LENGTH_LONG).show();
            }
        } catch (Exception e) {
            Toast.makeText(this, "Something went wrong", Toast.LENGTH_LONG)
                    .show();
        }
    }

    //returns to admin main activity
    public void returnToAdminMain()
    {
        Intent intent = new Intent(this, Admin_Main.class);
        intent.putExtra("json_data", json_string1);
        intent.putExtra("json_building_data", json_string2);
        startActivity(intent);
    }

    //calls php to insert new building in database
    public class BackgroundTask extends AsyncTask<String, Void, String> {

        Context ctx;
        BackgroundTask(Context ctx){
            this.ctx = ctx;
        }
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(String... params) {
            String insert_url = "http://compsci02.snc.edu/~philmt/csci460/create_building.php";
            //loads variables with sent parameters
            String name = params[0];
            String nickname = params[1];
            String lat = params[2];
            String log = params[3];
            String url2 = params[4];
            String address = params[5];
            String description = params[6];

                try{
                    URL url = new URL(insert_url);
                    //opens http connection, output stream, and buffer
                    HttpURLConnection httpURLConnection = (HttpURLConnection)url.openConnection();
                    httpURLConnection.setRequestMethod("POST");
                    httpURLConnection.setDoOutput(true);
                    OutputStream OS = httpURLConnection.getOutputStream();
                    BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(OS, "UTF-8"));
                    String data = URLEncoder.encode("building_name", "UTF-8") +"="+URLEncoder.encode(name, "UTF-8")+"&"+
                            URLEncoder.encode("building_nickname", "UTF-8") +"="+URLEncoder.encode(nickname, "UTF-8")+"&"+
                            URLEncoder.encode("building_lat", "UTF-8") +"="+URLEncoder.encode(lat, "UTF-8")+"&"+
                            URLEncoder.encode("building_long", "UTF-8") +"="+URLEncoder.encode(log, "UTF-8")+"&"+
                            URLEncoder.encode("building_weblink", "UTF-8") +"="+URLEncoder.encode(url2, "UTF-8")+"&"+
                            URLEncoder.encode("building_address", "UTF-8") +"="+URLEncoder.encode(address, "UTF-8")+"&"+
                            URLEncoder.encode("building_description", "UTF-8") +"="+URLEncoder.encode(description, "UTF-8")+"&"+
                            URLEncoder.encode("building_type_id", "UTF-8") +"="+URLEncoder.encode(building_type, "UTF-8")+"&"+
                            URLEncoder.encode("building_pic", "UTF-8") +"="+URLEncoder.encode(imgDecodableString, "UTF-8");
                    bufferedWriter.write(data);
                    bufferedWriter.flush();

                    //closes buffer and output stream
                    bufferedWriter.close();
                    OS.close();
                    InputStream IS = httpURLConnection.getInputStream();
                    IS.close();
                    return "Registration Success...";
                } catch (MalformedInputException e){
                    e.printStackTrace();
                } catch (IOException e){
                    e.printStackTrace();
                }


            return null;
        }

        @Override
        protected void onProgressUpdate(Void... values) {
            super.onProgressUpdate(values);
        }

        @Override
        protected void onPostExecute(String result) {
            //Toast.makeText(ctx, result, Toast.LENGTH_LONG).show();
            //retrieves updated json_string after new building has been inserted into the database
            BackgroundTask1 backgroundTask1 = new BackgroundTask1();
            backgroundTask1.execute();
        }
    }

    //retrieves updated json_string after new building has been inserted into the database
    class BackgroundTask1 extends AsyncTask<Void, Void, String> {
        String JSON_String;

        @Override
        protected String doInBackground(Void... params) {
            try {
                URL url = new URL("http://compsci02.snc.edu/~philmt/csci460/building_info.php");

                //opens http connection, input stream, and buffer
                HttpURLConnection httpURLConnection = (HttpURLConnection) url.openConnection();
                InputStream inputStream = httpURLConnection.getInputStream();
                BufferedReader bufferReader = new BufferedReader(new InputStreamReader(inputStream));
                StringBuilder stringBuilder = new StringBuilder();
                while ((JSON_String = bufferReader.readLine()) != null) {
                    stringBuilder.append(JSON_String + "\n");
                }

                //closes buffer, input stream, and http connection
                bufferReader.close();
                inputStream.close();
                httpURLConnection.disconnect();
                json_string2 = stringBuilder.toString().trim();
                return stringBuilder.toString().trim();
            } catch (MalformedInputException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            return null;
        }

        @Override
        protected void onProgressUpdate(Void... values) {
            super.onProgressUpdate(values);
        }

        @Override
        protected void onPostExecute(String result) {
            //sets json_string for buildings
            json_string2 = result;
            //returns to admin main activity
            returnToAdminMain();
        }
    }

}
